const getHeight = ({ numGenerations, circleDiameter, generationGap }) =>
  numGenerations * circleDiameter + (numGenerations - 1) * generationGap;

export const getSettings = ({ main, collateral }) => {
  const circleRadius = 1.5;
  const collateralCircleRadius = 1;
  const circleStrokeWidth = 0.25;
  const collateralCircleStrokeWidth = 0.2;
  const circleDiameter = circleRadius * 2 + circleStrokeWidth * 2;

  const lineStrokeWidth = circleStrokeWidth;
  const collateralLineStrokeWidth = collateralCircleStrokeWidth;

  const generationGap = 0.5;

  const width = (circleDiameter + lineStrokeWidth) * 4;

  const mainLineX = width - circleDiameter - lineStrokeWidth * 2;
  const collateralLineX = circleDiameter + collateralLineStrokeWidth * 2;

  const black = "#000";

  return {
    size: {
      width,
      height: {
        main: getHeight({
          numGenerations: main.length,
          circleDiameter,
          generationGap,
        }),
        collateral: getHeight({
          numGenerations: collateral.length,
          circleDiameter,
          generationGap,
        }),
      },
      generation: {
        gap: generationGap,
        height: circleDiameter + generationGap,
      },
    },
    circle: {
      main: {
        radius: circleRadius,
        diameter: circleDiameter,
        strokeColor: black,
        strokeWidth: circleStrokeWidth,
      },
      collateral: {
        radius: collateralCircleRadius,
        strokeColor: black,
        strokeWidth: collateralCircleStrokeWidth,
      },
    },
    line: {
      main: {
        strokeColor: black,
        strokeWidth: lineStrokeWidth,
      },
      collateral: {
        strokeColor: black,
        strokeWidth: collateralLineStrokeWidth,
      },
    },
    text: {
      main: {
        fill: black,
        size: 1.5,
      },
      collateral: {
        fill: black,
        size: 1,
      },
    },
    position: {
      main: {
        line: mainLineX,
        father: mainLineX - circleRadius - lineStrokeWidth,
        mother: mainLineX + circleRadius + lineStrokeWidth,
      },
      collateral: {
        line: collateralLineX,
        father:
          collateralLineX - collateralCircleRadius - collateralLineStrokeWidth,
        mother:
          collateralLineX + collateralCircleRadius + collateralLineStrokeWidth,
      },
    },
  };
};

export const getSettingsByLine = (settingsSection, collateral) =>
  collateral ? settingsSection.collateral : settingsSection.main;

export const getStrokeSettings = ({ settings, collateral }) => ({
  stroke: getSettingsByLine(settings.line, collateral).strokeColor,
  "stroke-width": getSettingsByLine(settings.line, collateral).strokeWidth,
});
