import * as fs from "node:fs";
import * as path from "node:path";
import { loadFile, saveFile } from "./files.js";
import { parse } from "yaml";

const original = "data/";

async function getFiles(dir) {
  const dirents = await fs.readdirSync(dir, { withFileTypes: true });

  const files = await Promise.all(
    dirents.map((dirent) => {
      const res = path.resolve(dir, dirent.name);
      return dirent.isDirectory() ? getFiles(res) : res;
    })
  );

  return files.flat();
}

const files = await getFiles("data/");

const renderSpouse = (spouse) => {
  let output = "";
  if (spouse.name) output += `name: ${spouse.name}\n`;
  if (spouse.date) output += `date: ${spouse.date}\n`;
  if (spouse.link) output += `link: ${spouse.link}\n`;

  if (spouse.children) {
    output += "[.children]\n";

    for (let grandchild of spouse.children) {
      output += `year: ${grandchild.year}\n`;

      if (grandchild.gender) {
        output += `gender: ${grandchild.gender}\n`;
      }
    }

    output += "[]\n";
  }

  return output;
};

const convertToArchieML = (obj, notes) => {
  const key = Object.keys(obj)[0];
  const family = obj[key];

  let output = `key: ${key}\n`;

  if (family.line) {
    output += `line: ${family.line}\n`;
  }

  output += "\n";

  // TODO other spouse for parents

  if (family.father) {
    const father = family.father;
    output += "{father}\n";
    if (father.name) output += `name: ${father.name}\n`;
    if (father.id) output += `id: ${father.id}\n`;
    if (father.birth)
      output += `birth: ${[father.birth.date, father.birth.place].join(
        " in "
      )}\n`;
    if (father.death)
      output += `death: ${[father.death.date, father.death.place].join(
        " in "
      )}\n`;

    if (father.parents) {
      output += "{.parents}\n";

      if (father.parents.father) output += `father: ${father.parents.father}\n`;
      if (father.parents.mother) output += `mother: ${father.parents.mother}\n`;
      if (father.parents.link) output += `link: ${father.parents.link}\n`;
    }

    if (father.spouses) {
      output += "{.spouses}\n";

      for (let spouse of father.spouses) {
        output += renderSpouse(spouse);
      }

      output += "[]\n";
    }

    output += "\n";
  }

  if (family.mother) {
    const mother = family.mother;
    output += "{mother}\n";
    if (mother.name) output += `name: ${mother.name}\n`;
    if (mother.id) output += `id: ${mother.id}\n`;
    if (mother.birth)
      output += `birth: ${[mother.birth.date, mother.birth.place].join(
        " in "
      )}\n`;
    if (mother.death)
      output += `death: ${[mother.death.date, mother.death.place].join(
        " in "
      )}\n`;

    if (mother.parents) {
      output += "{.parents}\n";

      if (mother.parents.father) output += `father: ${mother.parents.father}\n`;
      if (mother.parents.mother) output += `mother: ${mother.parents.mother}\n`;
      if (mother.parents.link) output += `link: ${mother.parents.link}\n`;
    }

    if (mother.spouses) {
      output += "{.spouses}\n";

      for (let spouse of mother.spouses) {
        output += renderSpouse(spouse);
      }

      output += "[]\n";
    }

    output += "\n";
  }

  if (family.marriage) {
    output += "{marriage}\n";
    if (family.marriage.date) output += `date: ${family.marriage.date}\n`;
    if (family.marriage.place) output += `place: ${family.marriage.place}\n`;

    output += "\n";
  }

  if (family.children) {
    output += "[children]\n";
    output += "\n";

    for (let child of family.children) {
      output += `name: ${child.name}\n`;

      if (child.id) output += `id: ${child.id}\n`;
      if (child.gender) output += `gender: ${child.gender}\n`;

      if (child.birth) {
        output += `birth: ${[child.birth.date, child.birth.place].join(
          " in "
        )}\n`;
      }

      if (child.death) {
        output += `death: ${[child.death.date, child.death.place].join(
          " in "
        )}\n`;
      }

      if (child.spouses) {
        output += "[.spouses]\n";

        for (let spouse of child.spouses) {
          output += renderSpouse(spouse);
        }

        output += "[]\n";
      }

      output += "\n";
    }

    output += "[]\n";
    output += "\n";
  }

  if (notes) {
    output += `notes: ${notes.trim()}\n:end`;
  }

  return output;
};

const processFile = (filename) => {
  console.log(filename);

  const dirname = path.dirname(filename);
  const newDirname = dirname.replace("/famish/data/", "/famish/newdata/");
  fs.mkdirSync(newDirname, { recursive: true });

  const basename = path.basename(filename);
  const outputBasename = basename.replace(".yaml", ".aml");

  const raw = loadFile(filename);
  const [yamlString, notesString] = raw.split("----");

  const obj = parse(yamlString);

  const output = convertToArchieML(obj, notesString);

  saveFile(path.join(newDirname, outputBasename), output);
};

files.forEach((filename) => processFile(filename));
