import * as childProcess from "node:child_process";
import * as fs from "node:fs";
import * as path from "node:path";

export const loadFile = (filename) => {
  try {
    return fs.readFileSync(filename, "utf8");
  } catch (error) {
    console.error(error);
    return;
  }
};

export const saveFile = (filename, data) => {
  try {
    return fs.writeFileSync(filename, data, "utf8");
  } catch (error) {
    console.error(error);
    return;
  }
};

export const getFiles = async (dir) => {
  const entries = await fs.readdirSync(dir, { withFileTypes: true });

  const files = await Promise.all(
    entries.map((entry) => {
      const res = path.resolve(dir, entry.name);
      return entry.isDirectory() ? getFiles(res) : res;
    })
  );

  return files.flat();
};

export const exportPDF = ({ command, path, filename }) => {
  childProcess.execSync(`${command} ${path}`, { stdio: "ignore" });
  fs.renameSync("output.pdf", filename);
};
