import { circle, line, path, rect, vline } from "../svg.js";

export const renderMarkerLine = (x, y, markerSettings) => [
  vline(
    x - markerSettings.style["stroke-width"] / 2,
    y - markerSettings.height / 2,
    y + markerSettings.height / 2,
    markerSettings.style
  ),
];

export const renderMarkerX = (x, y, markerSettings) => [
  line(
    x - markerSettings.width / 2,
    y - markerSettings.height / 2,
    x + markerSettings.width / 2,
    y + markerSettings.height / 2,
    markerSettings.style
  ),
  line(
    x + markerSettings.width / 2,
    y - markerSettings.height / 2,
    x - markerSettings.width / 2,
    y + markerSettings.height / 2,
    markerSettings.style
  ),
];

export const renderMarkerRectangle = (x, y, markerSettings) => [
  rect(
    x - markerSettings.width / 2,
    y - markerSettings.height / 2,
    markerSettings.width,
    markerSettings.height,
    markerSettings.style
  ),
];

export const renderMarkerDiamond = (x, y, markerSettings) => [
  path(
    `M${x - markerSettings.size} ${y} L${x} ${y - markerSettings.size} L${
      x + markerSettings.size
    } ${y} L${x} ${y + markerSettings.size} Z`,
    markerSettings.style
  ),
];

export const renderMarkerCircle = (x, y, markerSettings) => [
  circle(x, y, markerSettings.radius, markerSettings.style),
];

const renderMarker = (marker, y, settings) => {
  const markerSettings =
    settings.markers[marker.type] ?? settings.markers["default"];
  const markerFn = markerSettings.fn;

  return markerFn(marker.x, y, {
    ...markerSettings,
    style: {
      ...markerSettings.style,
      class: `marker-${marker.type}`,
    },
  });
};

export const renderMarkers = (markers, y, settings) =>
  (markers ?? []).reduce(
    (markers, marker) => [...markers, ...renderMarker(marker, y, settings)],
    []
  );

export default renderMarkers;
