const monthAbbrevToInt = (month) => {
  const months = [
    "Jan",
    "Feb",
    "Mar",
    "Apr",
    "May",
    "Jun",
    "Jul",
    "Aug",
    "Sep",
    "Oct",
    "Nov",
    "Dec",
  ];

  const monthInt = months.indexOf(month);

  return monthInt !== -1 ? monthInt + 1 : undefined;
};

const getDayMonthYear = (day, month, year) => ({
  day: day ? parseInt(day, 10) : undefined,
  month: monthAbbrevToInt(month ?? "Jun"), // default to mid-year if not present
  year: year ? parseInt(year, 10) : undefined,
});

const matchSingleDate = (dateString) => {
  const match = dateString
    .trim()
    .match(
      /(?<day>[\d]+)?\s*(?<month>Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)?\s*(?<year>\d{4})/
    );

  return match ? match.groups : {};
};

const parseDateString = (dateString) => {
  // Most likely this, do it and save it
  const { day, month, year } = matchSingleDate(dateString);
  const date = getDayMonthYear(day, month, year);

  // Between
  if (dateString.includes("between")) {
    const match = dateString
      .trim()
      .match(
        /between\s*(?<day1>[\d]+)?\s*(?<month1>Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Nov|Dec)?\s*(?<year1>\d{4})\s*and\s*(?<day2>[\d]+)?\s*(?<month2>Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Nov|Dec)?\s*(?<year2>\d{4})/
      );

    const { day1, month1, year1, day2, month2, year2 } = match.groups;

    return {
      after: getDayMonthYear(day1, month1, year1),
      before: getDayMonthYear(day2, month2, year2),
    };
  }

  // About
  if (dateString.includes("about")) {
    return {
      after: { ...date, year: date.year - 1 },
      before: { ...date, year: date.year + 1 },
      year: date.year,
    };
  }

  // Before
  if (dateString.includes("before")) {
    return {
      after: { ...date, year: date.year - 5 },
      before: date,
    };
  }

  // After
  if (dateString.includes("after")) {
    return {
      after: date,
      before: { ...date, year: date.year + 5 },
    };
  }

  // Single date
  return date;
};

const parsePlaceString = (placeString) => placeString.trim();

export const parseEventString = (eventString) => {
  const parts = eventString.split(" in ").map((s) => s.trim());

  return {
    ...(parts[0] && parseDateString(parts[0])),
    ...(parts.length > 1 && parts[1]
      ? { place: parsePlaceString(parts[1]) }
      : {}),
  };
};

export const parseYearFromEvent = (eventString) =>
  parseEventString(eventString).year;
