import {
  renderMarkerRectangle,
  renderMarkerLine,
  renderMarkerCircle,
  renderMarkerDiamond,
  renderMarkerX,
} from "./markers.js";

const black = "#000";
const white = "#fff";

export const getSettings = () => {
  return {
    size: {
      width: 550,
      xFactor: 3.8,
    },
    padding: {
      bottom: 10,
      left: 30,
      top: 5,
    },
    tracks: {
      stroke: black,
      "stroke-width": 0.5,
      parent: {
        height: 10,
        padding: 3,
      },
      child: {
        height: 8,
        padding: 1,
      },
      default: {
        height: 10,
        padding: 1,
      },
    },
    bands: {
      stroke: black,
      "stroke-width": 0.5,
    },
    markers: {
      tick: {
        height: 1.5,
        fn: renderMarkerLine,
        style: {
          "stroke-width": 0.5,
          stroke: black,
        },
      },
      birth: {
        height: 5,
        fn: renderMarkerLine,
        style: {
          "stroke-width": 0.5,
          stroke: black,
        },
      },
      death: {
        height: 5,
        fn: renderMarkerLine,
        style: {
          "stroke-width": 0.5,
          stroke: black,
        },
      },
      marriage: {
        width: 1,
        height: 10,
        fn: renderMarkerRectangle,
        style: {
          fill: black,
        },
      },
      divorce: {
        width: 4,
        height: 7,
        fn: renderMarkerX,
        style: {
          "stroke-width": 0.75,
          stroke: black,
        },
      },
      son: {
        size: 2.25,
        fn: renderMarkerDiamond,
        style: {
          "stroke-width": 0.65,
          stroke: black,
          fill: white,
        },
      },
      daughter: {
        radius: 2,
        fn: renderMarkerCircle,
        style: {
          "stroke-width": 0.65,
          stroke: black,
          fill: black,
        },
      },
      child: {
        radius: 2,
        fn: renderMarkerCircle,
        style: {
          "stroke-width": 0.65,
          stroke: black,
          fill: white,
        },
      },
      default: {
        radius: 1,
        fn: renderMarkerCircle,
        style: {
          "stroke-width": 0.5,
          stroke: "#f00",
          fill: "#00f",
        },
      },
    },
    labels: {
      xOffset: -6,
      yOffset: 0.5,
      style:
        "font-weight: bold; font-size: 6pt; letter-spacing: 0.05rem; alignment-baseline: middle; text-anchor: end;",
    },
  };
};
