import { renderMarkers } from "./markers.js";
import { hline, text, vline } from "../svg.js";

const renderSegment = (segment, y, settings) =>
  hline(segment.start, segment.end, y, {
    class: `track-${segment.type}`,
    stroke: settings.tracks.stroke,
    "stroke-width": settings.tracks["stroke-width"],
    ...(segment.type === "dotted" ? { "stroke-dasharray": "2 2" } : {}),
  });

const renderLine = (segments, y, settings) =>
  (segments ?? []).map((segment) => renderSegment(segment, y, settings));

const renderLabel = (track, y, settings) => [
  ...(track.label
    ? [
        text(
          (track.segments?.[0].start ?? 0) + settings.labels.xOffset,
          y + settings.labels.yOffset,
          track.label,
          {
            style: settings.labels.style,
          }
        ),
      ]
    : []),
];

const renderBand = (band, tracks, settings) =>
  vline(
    band.x - settings.bands["stroke-width"] / 2,
    tracks[band.start],
    tracks[band.end],
    {
      class: `band-${band.type}`,
      stroke: settings.bands.stroke,
      "stroke-width": settings.bands["stroke-width"],
      ...(band.type === "dotted" ? { "stroke-dasharray": "3 3" } : {}),
    }
  );

const renderBands = (bands, tracks, settings) =>
  (bands ?? []).map((band) => renderBand(band, tracks, settings));

const renderTrack = (track, y, settings) =>
  `<g class="track">${[
    ...renderLine(track.segments, y, settings),
    ...renderMarkers(track.markers, y, settings),
    ...renderLabel(track, y, settings),
  ]
    .filter((part) => part)
    .join("\n")}</g>`;

const trackHeight = (track, settings) => {
  const trackSettings =
    settings.tracks[track.type] ?? settings.tracks["default"];

  const height = track.height ?? trackSettings?.height ?? 500;
  const padding = track.padding ?? trackSettings?.padding ?? 100;

  return height + padding + padding;
};

export const renderTracksAndBands = (tracks, bands, settings) => {
  const outputTracks = tracks.reduce(
    (results, track) => {
      const trackType =
        settings.tracks[track.type] ?? settings.tracks["default"];

      return {
        output: [
          ...results.output,
          renderTrack(track, results.y + trackType.padding, settings),
        ],
        y: results.y + trackHeight(track, settings),
        height: results.height + trackHeight(track, settings),
        tracks: [...results.tracks, results.y + trackType.padding],
      };
    },
    {
      y: settings.padding?.top ?? 0,
      tracks: [],
      output: [],
      height: 0,
    }
  );

  return {
    content: `<g class="sparklines" transform="translate(${
      settings.padding.left
    } 0)">
    ${[
      ...outputTracks.output,
      ...renderBands(bands, outputTracks.tracks, settings),
    ].join("\n")}`,
    height: outputTracks.height + (settings.padding.bottom ?? 0),
  };
};

export default renderTracksAndBands;
