export const parseAttrs = (attrs) =>
  Object.keys(attrs)
    .filter((attr) => attr !== "fn")
    .map((attr) => `${attr}="${attrs[attr]}"`)
    .join(" ");

export const circle = (x, y, r, attrs) =>
  `<circle ${parseAttrs({ cx: x, cy: y, r, ...attrs })} />`;

export const line = (x1, y1, x2, y2, attrs) =>
  `<line ${parseAttrs({ x1, y1, x2, y2, ...attrs })} />`;

export const vline = (x, y1, y2, attrs) =>
  `<line ${parseAttrs({ x1: x, y1, x2: x, y2, ...attrs })} />`;

export const hline = (x1, x2, y, attrs) =>
  `<line ${parseAttrs({ x1, y1: y, x2, y2: y, ...attrs })} />`;

export const path = (d, attrs) => `<path ${parseAttrs({ d, ...attrs })} />`;

export const rect = (x, y, width, height, attrs) =>
  `<rect ${parseAttrs({ x, y, width, height, ...attrs })} />`;

export const text = (x, y, text, attrs) =>
  `<text ${parseAttrs({ x, y, ...attrs })}>${text}</text>`;

export const svg = ({ width, height, className, content }) =>
  `<svg ${parseAttrs({
    class: className,
    viewBox: `0 0 ${width} ${height}`,
    xmlns: "http://www.w3.org/2000/svg",
  })}>${content}</svg>`;
